/*****************************************************************************
*       Filename:       SPEECH.C                                             *
******************************************************************************
*       Author:         Rodger Richey                                        *
*       Title:          Senior Applications Engineer                         *
*       Company:        Microchip Technology Incorporated                    *
*       Revision:       0                                                    *
*       Date:           1-9-96                                               *
*       Compiled using Bytecraft Ltd. MPC version BC.193                     *
******************************************************************************
*       Include files:                                                       *
*               16c73.h - Standard include file for the PIC16C73 device      *
*               delay14.h - Standard include file for delay routines         *
*               adpcm.h - Include file with ADPCM related information (Rev0) *
*               adpcm.c - Source file with ADPCM routines (Rev0)             *
******************************************************************************
*       This file contains the routines to:                                  *
*               - Initialize the PIC16C73                                    *
*               - Read data from the EPROMs                                  *
*               - Call the ADPCM decoder routine                             *
*               - Send data to the DAC and PWM                               *
*               - Increment the address of the EPROMs                        *
******************************************************************************
*       External Clock Frequency:       20MHz                                *
*       Configuration Bit Settings                                           *
*                                       OSC:    HS                           *
*                                       WDT:    Off                          *
*                                       PWRTE:  Off                          *
*                                       CP:     Off                          *
*       Program Memory Usage:           1030 words                           *
*       Data Memory Usage:              24 bytes                             *
*****************************************************************************/
#pragma option v
#include <16c63.h>
#include "adpcm.h"

// Defines for PORTA
#define CE   0          // Chip Enable for EPROMS
#define OE1  1          // Output Enable for EPROM #1
#define OE2  2          // Output Enable for EPROM #2

// Defines for PORTC
#define STRT 0          // Start Button input
#define PWM  2          // PWM output
#define RST  6          // EPROM counter reset
#define ECLK 7          // EPROM counter clock

unsigned long Address;  // EPROM address
unsigned char Wait;
struct ADPCMState state;// ADPCM structure

#include "adpcm.c"

/*****************************************************************************
*       Init73 - This function initializes PORTA, PORTC, and PWM             *
******************************************************************************
*       Input Variables:                                                     *
*               None                                                         *
*       Return Variables:                                                    *
*               None                                                         *
*****************************************************************************/
void Init73(void)
{
	OPTION = 8;
//	ADCON1 = 7;     // Make PORTA all digital
	PORTA = 7;      // Set CE,OE1,OE2
	TRISA = 0;      // Make PORTA all outputs
	PORTC = 0x40;   // Set ECLK, all outputs
	TRISC = 0x05;   // Clear RST output, STRT input
//	PR2 = 155;      // Set PWM for 32kHz period
//	T2CON = 0x18;   // Set Timer2 postscaler to 4
//	PR2 = 155;		// Set PWM for 8kHz period
//	T2CON = 0b00000001;
	PR2 = 77;		// Set PWM for 16KHz period
	T2CON = 0b00001001;
	CCPR1L = 0;
	return;
}

/*****************************************************************************
*       main - This function controls everything, kinda like a god.          *
******************************************************************************
*       Input Variables:                                                     *
*               None                                                         *
*       Return Variables:                                                    *
*               None                                                         *
*****************************************************************************/
void main(void)
{

	char code;                      // EPROM byte
	signed long sample;             // decoded sample
	unsigned long pwmout;           // temporary variable for PWM
 
	Init73();                       // Initialize the PIC16C73

	Wait = 1;

	PORTC.RST = 1;                  // Reset the EPROM counter
	NOP();
	NOP();
	PORTC.RST = 0;
	NOP();
	NOP();
	PORTA.OE1 = 0;                  // Enable EPROM #1
	NOP();
	NOP();
	PORTA.CE = 0;
	NOP();
	NOP();

	PIR1.TMR2IF = 0;                // Enable Timer2 Interrupts
	PIE1.TMR2IE = 1;
	INTCON.PEIE = 1;                // Enable PEIE and GIE
	INTCON.GIE = 1; 

	while(1)
	{
		while(PORTC.STRT);      // Wait for Start button to be
		while(!PORTC.STRT);     // pressed then released

		Address = 0;            // Clear EPROM address
		state.prevsample = 0;   // Clear ADPCM structure
		state.previndex = 0;

		TRISC.PWM = 0;          // Make PWM an output
		T2CON.TMR2ON = 1;       // Enable Timer2
		CCP1CON = 0x0f;         // Enable PWM

		do
		{
			// Read ADPCM code byte from EPROMs
			code = PORTB;

			// Increment Address of EPROMs
			Address++;
			PORTC.ECLK = 1;
			NOP();
			NOP();
			PORTC.ECLK = 0;
			NOP();
			NOP();

			// Decode upper 4 bits of code
			sample = ADPCMDecoder( (code>>4)&0x0f );
			while(Wait);    // Wait for 8kHz to output
			Wait = 1;
			pwmout = sample;          // Write to PWM
			if(sample<0)              // Add offset to
				pwmout -= 0x8000; // sample
			else
				pwmout += 0x8000;
			CCPR1L = (pwmout>>9)&0x007f; // Write 7 bits
			CCP1CON &= 0xcf;             // to CCPR1L
			if(pwmout&0x0100)            // Write 2 bits
				CCP1CON.5 = 1;       // to CCP1CON
			if(pwmout&0x0080)            // to get 9-bit
				CCP1CON.4 = 1;       // PWM

			// Decode lower 4 bits of code
			sample = ADPCMDecoder(code&0x0f);
			while(Wait);    // Wait for 8kHz to output
			Wait = 1;
			pwmout = sample;          // Write to PWM
			if(sample<0)              // Add offset to
				pwmout -= 0x8000; // sample
			else
				pwmout += 0x8000;
			CCPR1L = (pwmout>>9)&0x007f; // Write 7 bits
			CCP1CON &= 0xcf;             // to CCPR1L
			if(pwmout&0x0100)            // Write 2 bits
				CCP1CON.5 = 1;       // to CCP1CON
			if(pwmout&0x0080)            // to get 9-bit
				CCP1CON.4 = 1;       // PWM

		} while(Address);       // Has Address overflowed?

		PORTA.RST = 1;          // Reset EPROM counter
		NOP();
		NOP();
		PORTA.RST = 0;
		NOP();
		NOP();
		PORTA.CE = 1;           // Disable EPROM #1
		NOP();
		NOP();
		PORTA.OE1 = 1;
		NOP();
		NOP();
		PORTA.OE2 = 0;          // Enable EPROM #2
		NOP();
		NOP();
		PORTA.CE = 0;
		NOP();
		NOP();
		Address = 0;            // Clear EPROM Address
		NOP();
		NOP();

		do
		{
			// Read ADPCM code byte from EPROMs
			code = PORTB;

			// Increment Address of EPROMS
			Address++;
			PORTC.ECLK = 1;
			NOP();
			NOP();
			PORTC.ECLK = 0;
			NOP();
			NOP();

			// Decode upper 4 bits of code
			sample = ADPCMDecoder( (code>>4)&0x0f );
			while(Wait);    // Wait for 8kHz
			Wait = 1;
			pwmout = sample;          // Write to PWM
			if(sample<0)              // Add offset to
				pwmout -= 0x8000; // sample
			else
				pwmout += 0x8000;
			CCPR1L = (pwmout>>9)&0x007f; // Write 7 bits
			CCP1CON &= 0xcf;             // to CCPR1L
			if(pwmout&0x0100)            // Write 2 bits
				CCP1CON.5 = 1;       // to CCP1CON
			if(pwmout&0x0080)            // to get 9-bit
				CCP1CON.4 = 1;       // PWM

			// Decode lower 4 bits of code
			sample = ADPCMDecoder(code&0x0f);
			while(Wait);    // Wait for 8kHz
			Wait = 1;
			pwmout = sample;          // Write to PWM
			if(sample<0)              // Add offset to
				pwmout -= 0x8000; // sample
			else
				pwmout += 0x8000;
			CCPR1L = (pwmout>>9)&0x007f; // Write 7 bits
			CCP1CON &= 0xcf;             // to CCPR1L
			if(pwmout&0x0100)            // Write 2 bits
				CCP1CON.5 = 1;       // to CCP1CON
			if(pwmout&0x0080)            // to get 9-bit
				CCP1CON.4 = 1;       // PWM

		} while(Address);       // Has Address overflowed?

		T2CON.TMR2ON = 0;       // Disable Timer2
		CCP1CON = 0;            // Disable PWM
		TRISC.PWM = 1;          // Make PWM input

		PORTA.CE = 1;           // Disable EPROM #2
		NOP();
		NOP();
		PORTA.OE2 = 1;
		NOP();
		NOP();
		PORTA.RST = 1;          // Reset EPROM counter
		NOP();
		NOP();
		PORTA.RST = 0;
		NOP();
		NOP();
		PORTA.OE1 = 0;          // Enable EPROM #1
		NOP();
		NOP();
		PORTA.CE = 0;
	}

}

/*****************************************************************************
*       __INT - This is the interrupt service routine. Only Timer 2 overflow *
*               is implemented.                                              *
******************************************************************************
*       Input Variables:                                                     *
*               None                                                         *
*       Return Variables:                                                    *
*               None                                                         *
*****************************************************************************/
void __INT(void)
{
	if(PIR1.TMR2IF)                 // Timer2 overflow interrupt
	{
		Wait = 0;
		PIR1.TMR2IF = 0;        // Clear flag
	}
	return;
}


